/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.awt.*;
import java.awt.event.*;

public class NvDrawerJ2D extends NvDrawer {

    private Canvas canvas;
    private Image doubleBuffer;
    private int xbase, ybase;

    public NvDrawerJ2D(NvChart chart, int xgrid, int ygrid) {
        super(chart, xgrid, ygrid);
        xbase = chart.gridDimension() * xgrid / 2;
        ybase = chart.gridDimension() * ygrid / 2;
    }

    public void update() {
        if (doubleBuffer == null) return;
        Graphics2D g2 = (Graphics2D)doubleBuffer.getGraphics();
        drawView(g2);
        canvas.repaint();
    }

    public void initDrawer() throws Exception {
        canvas = new Canvas() {
            private static final long serialVersionUID = 0;

            public void update(Graphics g) {
                paint(g);
            }
            
            public void paint(Graphics g) {
                g.drawImage(doubleBuffer, 0, 0, canvas);
            }

            public void setBounds(int x, int y, int w, int h) {
                doubleBuffer = createImage(w, h);
                super.setBounds(x, y, w, h);
            }
        };
        canvas.setBounds(0, 0, 600, 600);
        frame.getContentPane().add(canvas, BorderLayout.CENTER);
        canvas.setVisible(true);
        canvas.addKeyListener(this);
        canvas.addMouseListener(this);
        canvas.requestFocus();
        doubleBuffer = canvas.createImage(canvas.getWidth(), canvas.getHeight());
    }

    public void drawView(Graphics2D g2) {
        g2.setColor(backColor);
        g2.fillRect(0, 0, canvas.getWidth(), canvas.getHeight());
        g2.translate(xbase, ybase);

        for (NvNode node: chart.getNodes()) {
            for (NvLink link: node.getLinks()) {
                drawLink(g2, link);
            }
        }
        for (NvNode node: chart.getNodes()) {
            if (node != chart.getFocus())
                drawNode(g2, node);
        }
        drawNode(g2, chart.getFocus());
    }

    protected void drawNode(Graphics2D g2, NvNode node) {
        int x = (int)(node.x * xgrid);
        int y = (int)(node.y * ygrid);

        if (node.renderingMode == NvNode.THUMBNAIL) {
            node.renderingCache = node.thumbnail;
            if (g2.drawImage((Image)node.renderingCache, x, y,
                    xgrid, ygrid, imageObserver)) {
                g2.setColor(labelColor);
                g2.drawRoundRect(x, y, xgrid, ygrid, 8, 8);
                return;
            }
        }
        Image image = (Image)node.renderingCache;
        if (image == null || image.getWidth(imageObserver) != xgrid
                && image.getHeight(imageObserver) != ygrid) {
            node.renderingCache = image =  
                makeLabelImage(node.getLabel(0), xgrid, ygrid, node.lastScan >= 0);
        }
        g2.drawImage(image, x, y, xgrid, ygrid, imageObserver);
    }

    protected void drawLink(Graphics2D g2, NvLink link) {
        NvNode s = link.getSource();
        NvNode d = link.getDest();
        g2.setColor(linkColor);
        if (s == chart.getFocus()) g2.setColor(linkColor2);
        g2.drawLine((int)(s.x * xgrid + xgrid / 2), (int)(s.y * ygrid + ygrid / 2),
                (int)(d.x * xgrid + xgrid / 2), (int)(d.y * ygrid + ygrid / 2));
    }

    public void keyTyped(KeyEvent e) {
        Dimension d;
        switch (e.getKeyChar()) {
        case 27:
            System.exit(0);
        case 'k':
            xbase -= 10;
            break;
        case 'j':
            xbase += 10;
            break;
        case 'm':
            ybase -= 10;
            break;
        case 'i':
            ybase += 10;
            break;
        case 'z':
            d = canvas.getSize();
            xbase = d.width / 2 - (d.width / 2 - xbase) / 2;
            ybase = d.height / 2 - (d.height / 2 - ybase) / 2;
            xgrid /= 2;
            ygrid /= 2;
            break;
        case 'Z':
            d = canvas.getSize();
            xbase = d.width / 2 - (d.width / 2 - xbase) * 2;
            ybase = d.height / 2 - (d.height / 2 - ybase) * 2;
            xgrid *= 2;
            ygrid *= 2;
            break;
        case 'l':
            chart.liftUp(chart.getFocus());
            break;
        case 'L':
            chart.liftDown(chart.getFocus());
            break;
        case 'x':
            chart.liftDownAll();
            break;
        }
        e.consume();
        update();
    }

    public void sliderChangedXT(int v) {
        xbase = chart.gridDimension() * xgrid / 2 - v * 10;
    }
    public void sliderChangedXR(int v) {
        //rotZ = v;        
    }
    public void sliderChangedYT(int v) {
        ybase = chart.gridDimension() * ygrid / 2 - v * 10;
    }
    public void sliderChangedYR(int v) {
        //rotX = v;
    }
    public void sliderChangedZ(int v) {
        xgrid = 64 / (int)((-v / 20.1f));
        ygrid = 32 / (int)((-v / 20.1f));        
    }

    public void sliderChangedLift(int v) {
        v = 5 - v;
        if (v < 0) return;
        NvNode node = chart.getFocus();
        if (node.doi < v)
            chart.liftUp(node, v);
        else if (node.doi > v)
            chart.liftDown(node, v);
    }
    
    public void mousePressed(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        // int b = e.getButton();

        NvNode found = null;
        for (NvNode node: chart.getNodes()) {
            if (node.x * xgrid + xbase < x && x < (node.x + 1) * xgrid + xbase &&
                node.y * ygrid + ybase < y && y < (node.y + 1) * ygrid + ybase) {
                found = node;
                break;
            }
        }
        if (found != null) {
            chart.setFocus(found);
            if (found.lastScan < 0)
                chart.startScanningLinks(found);
        }
        e.consume();
    }
}
